// Inspired by
// https://observablehq.com/@d3/parallel-coordinates
// https://observablehq.com/@d3/brushable-parallel-coordinates

import {
  select,
  transition,
  scaleLinear,
  scalePoint,
  scaleOrdinal,
  schemeCategory10,
  extent,
  line,
  brushY,
} from 'd3';
import { data } from '@eschuman20/footballscorers';
import { observeResize } from '@curran/responsive-axes';
import { parallelCoordinates } from './parallelCoordinates';

// Assign d.id to the index.
// This is used later for object constancy in transitions.
data.forEach((d, i) => {
  d.id = i;
});

export const main = (container, { state, setState }) => {
  // Initialize dimensions by observing resise.
  const dimensions = observeResize({
    state,
    setState,
    container,
  });
  if (dimensions === null) return;
  const { width, height } = dimensions;

  // Initialize the brushed intervals state to an empty object.
  if (!state.brushedIntervals) {
    setState((state) => ({
      ...state,
      brushedIntervals: {},
    }));
    return;
  }
  const { brushedIntervals } = state;

  // Updates the brushed intervals when brushes change.
  const updateBrushedInterval = ({ column, interval }) => {
    setState((state) => ({
      ...state,
      brushedIntervals: {
        ...state.brushedIntervals,
        [column]: interval,
      },
    }));
  };

  const columns = [
    state.x1Value,
    state.x2Value,
    state.x3Value,
    state.x4Value,
    state.x5Value
  ];
  
  const columnTypes = {
    Matches_Played: 'quantitative',
    Mins: 'quantitative',
    Shots: 'quantitative',
    OnTarget: 'quantitative',
    Goals: 'quantitative',
    Substitution: 'quantitative',
    Shots_Per_Avg_Match: 'quantitative',
    On_Target_Per_Avg_Match: 'quantitative',
    xG: 'quantitative',
    xG_Per_Avg_Match: 'quantitative'
  };
  const colorValue = (d) => d.League;
  const idValue = (d) => d.id;
  const linelabel = (d) => d['Player Names'];

  // Use this to inspect the brushedIntervals as they change!
  // console.log(
  //   JSON.stringify(brushedIntervals, null, 2),
  // );

  if (!state.x1Value)
    setState((prevState) => ({
      ...prevState,
      x1Value: 'Matches_Played',
    }));

  if (!state.x2Value)
    setState((prevState) => ({
      ...prevState,
      x2Value: 'Mins',
    }));

  if (!state.x3Value)
    setState((prevState) => ({
      ...prevState,
      x3Value: 'Shots',
    }));

  if (!state.x4Value)
    setState((prevState) => ({
      ...prevState,
      x4Value: 'OnTarget',
    }));
  
  if (!state.x5Value)
    setState((prevState) => ({
      ...prevState,
      x5Value: 'Goals',
    }));

  const x1Select = document.createElement('select');
  x1Select.innerHTML = `
    <option value="Matches_Played">Matches Played</option>
    <option value="Goals">Goals</option>
    <option value="Shots">Shots</option>
    <option value="Shots_Per_Avg_Match">Shots Per Match</option>
    <option value="OnTarget">Shots On Target</option>
    <option value="On_Target_Per_Avg_Match">Shots On Target Per Match</option>
    <option value="xG">xG Score</option>
    <option value="xG_Per_Avg_Match">xG Per Match</option>
    <option value="Substitution">Substitutions</option>
    <option value="Mins">Minutes</option>
  `;

  const x2Select = document.createElement('select');
  x2Select.innerHTML = `
    <option value="Mins">Minutes</option>
    <option value="Matches_Played">Matches Played</option>
    <option value="Goals">Goals</option>
    <option value="Shots">Shots</option>
    <option value="Shots_Per_Avg_Match">Shots Per Match</option>
    <option value="OnTarget">Shots On Target</option>
    <option value="On_Target_Per_Avg_Match">Shots On Target Per Match</option>
    <option value="xG">xG Score</option>
    <option value="xG_Per_Avg_Match">xG Per Match</option>
    <option value="Substitution">Substitutions</option>
  `;

  const x3Select = document.createElement('select');
  x3Select.innerHTML = `
    <option value="Shots">Shots</option>
    <option value="Mins">Minutes</option>
    <option value="Matches_Played">Matches Played</option>
    <option value="Goals">Goals</option>
    <option value="Shots_Per_Avg_Match">Shots Per Match</option>
    <option value="OnTarget">Shots On Target</option>
    <option value="On_Target_Per_Avg_Match">Shots On Target Per Match</option>
    <option value="xG">xG Score</option>
    <option value="xG_Per_Avg_Match">xG Per Match</option>
    <option value="Substitution">Substitutions</option>
  `;

  const x4Select = document.createElement('select');
  x4Select.innerHTML = `
    <option value="OnTarget">Shots On Target</option>
    <option value="Shots">Shots</option>
    <option value="Mins">Minutes</option>
    <option value="Matches_Played">Matches Played</option>
    <option value="Goals">Goals</option>
    <option value="Shots_Per_Avg_Match">Shots Per Match</option>
    <option value="On_Target_Per_Avg_Match">Shots On Target Per Match</option>
    <option value="xG">xG Score</option>
    <option value="xG_Per_Avg_Match">xG Per Match</option>
    <option value="Substitution">Substitutions</option>
  `;

  const x5Select = document.createElement('select');
  x5Select.innerHTML = `
    <option value="Goals">Goals</option>
    <option value="OnTarget">Shots On Target</option>
    <option value="Shots">Shots</option>
    <option value="Mins">Minutes</option>
    <option value="Matches_Played">Matches Played</option>
    <option value="Shots_Per_Avg_Match">Shots Per Match</option>
    <option value="On_Target_Per_Avg_Match">Shots On Target Per Match</option>
    <option value="xG">xG Score</option>    
    <option value="xG_Per_Avg_Match">xG Per Match</option>
    <option value="Substitution">Substitutions</option>
  `;

  x1Select.addEventListener('change', () => {
    setState((prevState) => ({
      ...state,
      x1Value: x1Select.value,
      x2Value: x2Select.value,
      x3Value: x3Select.value,
      x4Value: x4Select.value,
      x5Value: x5Select.value,
    }));
  });

  x2Select.addEventListener('change', () => {
    setState((prevState) => ({
      ...state,
      x1Value: x1Select.value,
      x2Value: x2Select.value,
      x3Value: x3Select.value,
      x4Value: x4Select.value,
      x5Value: x5Select.value,
    }));
  });
  x3Select.addEventListener('change', () => {
    setState((prevState) => ({
      ...state,
      x1Value: x1Select.value,
      x2Value: x2Select.value,
      x3Value: x3Select.value,
      x4Value: x4Select.value,
      x5Value: x5Select.value,
    }));
  });
  x4Select.addEventListener('change', () => {
    setState((prevState) => ({
      ...state,
      x1Value: x1Select.value,
      x2Value: x2Select.value,
      x3Value: x3Select.value,
      x4Value: x4Select.value,
      x5Value: x5Select.value,
    }));
  });
  x5Select.addEventListener('change', () => {
    setState((prevState) => ({
      ...state,
      x1Value: x1Select.value,
      x2Value: x2Select.value,
      x3Value: x3Select.value,
      x4Value: x4Select.value,
      x5Value: x5Select.value,
    }));
  });

  container.appendChild(x1Select);
  container.appendChild(x2Select);
  container.appendChild(x3Select);
  container.appendChild(x4Select);
  container.appendChild(x5Select);

  // Set up the SVG.
  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height)
    .style('background', '#090F10');

  // Invoke the parallel coordinates.
  svg.call(parallelCoordinates, {
    data,
    columns,
    columnTypes,
    colorValue,
    idValue,
    width,
    height,
    brushedIntervals,
    updateBrushedInterval,
    colorLegendX: width - 150,
    colorLegendY: height - 332,
    colorLegendLabel: 'League',
    linelabel,
  });
};
